@@ -1,5 +1,10 @@
 
-# $Id: Ebay.pm,v 2.255 2010-08-17 21:59:25 Martin Exp $
+package WWW::Search::Ebay;
+
+use strict;
+use warnings;
+
+our $VERSION = 2.275;
 
 =head1 NAME
 
@@ -53,9 +58,6 @@ field is a string containing the high bid or starting bid as a
 human-readable monetary value in seller-native units, e.g. "$14.95" or
 "GBP 6.00".
 
-In the resulting L<WWW::Search::Result> objects, the category() field
-contains the Ebay category number.
-
 In the resulting L<WWW::Search::Result> objects, the sold() field will
 be non-zero if the item has already sold.  (Only if you're using
 WWW::Search::Ebay::Completed)
@@ -67,7 +69,7 @@ the count of items matching your query in each category and
 subcategory.  (Special thanks to Nick Lokkju for this code!)  For
 example:
 
-  $oSearch->{category} = [
+  $oSearch->{categories} = [
           {
             'ID' => '1',
             'Count' => 19,
@@ -100,23 +102,6 @@ you will get back one WWW::Search::Result without bid or price information.
 
 =over
 
-=item Search descriptions
-
-To search titles and descriptions, add 'srchdesc'=>'y' to the query options:
-
-  $oSearch->native_query($sQuery, { srchdesc => 'y' } );
-
-=item Search one category
-
-To restrict your search to a particular eBay category,
-find out eBay's ID number for the category and
-add 'sacategory'=>123 to the query options:
-
-  $oSearch->native_query($sQuery, { sacategory => 48995 } );
-
-If you send a single asterisk or a single space as the query string,
-the results will be ALL the auctions in that category.
-
 =item Limit search by price range
 
 Contributed by Brian Wilson:
@@ -133,11 +118,6 @@ Contributed by Brian Wilson:
 
 =cut
 
-package WWW::Search::Ebay;
-
-use strict;
-use warnings;
-
 use base 'WWW::Search';
 
 use constant DEBUG_DATES => 0;
@@ -147,7 +127,7 @@ use Carp ();
 use CGI;
 use Data::Dumper;  # for debugging only
 use Date::Manip;
-&Date_Init('TZ=US/Pacific') unless (defined($ENV{TZ}) && ($ENV{TZ} ne ''));
+# Date_Init("setdate=now,America/Los_Angeles");
 use HTML::TreeBuilder;
 use LWP::Simple;
 use WWW::Search qw( generic_option strip_tags );
@@ -155,8 +135,6 @@ use WWW::Search qw( generic_option strip_tags );
 use WWW::SearchResult 2.072;
 use WWW::Search::Result;
 
-our
-$VERSION = do { my @r = (q$Revision: 2.255 $ =~ /\d+/g); sprintf "%d."."%03d" x $#r, @r };
 our $MAINTAINER = 'Martin Thurn <mthurn@cpan.org>';
 my $cgi = new CGI;
 
@@ -177,8 +155,14 @@ sub _native_setup_search
 
   $self->{'_next_to_retrieve'} = 0;
   $self->{'_num_hits'} = 0;
-
+  # As of 2013-03-01 (probably much before that, but first time I
+  # looked at it in quite a while): <p>To use our basic experience
+  # which does not require JavaScript, <a
+  # href="http://www.ebay.com/sch/i.html?LH_Auction=1&_nkw=trinidad+tobago+flag&_armrs=1&_from=&_ipg=50&_jsoff=1">click
+  # here</a>.</p>
+  $self->{search_host} ||= 'http://www.ebay.com'; # as of 2013-03-01
   $self->{search_host} ||= 'http://search.ebay.com';
+  $self->{search_path} ||= '/sch/i.html'; # as of 2013-03-01
   $self->{search_path} ||= '/ws/search/SaleSearch';
   if (!defined($self->{_options}))
     {
@@ -199,6 +183,20 @@ sub _native_setup_search
                          # user may have customized (which would come
                          # through via cookies):
                          socustoverride => 1,
+                         # Output basic HTML, not JavaScript:
+                         _armrs => 1,
+                        };
+    $self->{_options} = {
+                         _nkw => $native_query,
+                         _armrs => 1,
+                         # Turn off JavaScript:
+                         _jsoff => 1,
+                         # Search AUCTIONS ONLY:
+                         LH_Auction => 1,
+                         _ipg => $self->{_hits_per_page},
+                         # Which page are we on:
+                         # _from => 2,
+                         # 
                         };
     } # if
   if (defined($rhOptsArg))
@@ -365,6 +363,26 @@ sub _create_description
   return $sDesc;
   } # _create_description
 
+sub _parse_category
+  {
+  my $self = shift;
+  my $oTD = shift;
+  return -1 if ! ref $oTD;
+  my $oA = $oTD->look_down(_tag => 'a');
+  return -1 if ! ref $oA;
+  if (DEBUG_COLUMNS || (1 < $self->{_debug}))
+    {
+    my $s = $oA->as_HTML;
+    print STDERR " DDD   TDcategory's A ===$s===\n";
+    } # if
+  my $sURL = $oA->attr('href') || q{};
+  if ($sURL =~ m/sibeleafcat=(\d+)/)
+    {
+    return $1;
+    } # if
+  return -1;
+  } # _parse_category
+
 sub _parse_price
   {
   my $self = shift;
@@ -376,7 +394,7 @@ sub _parse_price
     {
     print STDERR " DDD   try TDprice ===$s===\n";
     } # if
-  if ($oTDprice->attr('class') =~ m'\bebcBid\b')
+  if ($oTDprice->attr('class') =~ m/\bebcBid\b/)
     {
     # If we see this, we must have been searching for Stores items
     # but we ran off the bottom of the Stores item list and ran
@@ -386,7 +404,7 @@ sub _parse_price
     # maybe just maybe we hit this because of a parsing glitch which
     # might correct itself on the next TD.
     } # if
-  if ($oTDprice->attr('class') !~ m'\b(ebcPr|prices|prc)\b')
+  if ($oTDprice->attr('class') !~ m/(ebcPr|prices?|prc)/)
     {
     # If we see this, we probably were searching for Store items
     # but we ran off the bottom of the Store item list and ran
@@ -394,15 +412,28 @@ sub _parse_price
     return 0;
     # There is a separate backend for searching Auction items!
     } # if
-  if ($oTDprice->look_down(_tag => 'span',
-                          class => 'ebSold'))
+  if (
+      $oTDprice->look_down(_tag => 'span',
+                          class => 'ebSold')
+      ||
+      $oTDprice->look_down(_tag => 'span',
+                          class => 'bold bidsold')
+     )
     {
     # This item sold, even if it had no bids (i.e. Buy-It-Now)
     $hit->sold(1);
     } # if
+  if (my $oChild = $oTDprice->look_down(_tag => 'div',
+                                        itemprop => 'price'))
+    {
+    # As of 2013-03, we need to separate out the price and the bid:
+    $oTDprice = $oChild;
+    } # if
   my $iPrice = $oTDprice->as_text;
   print STDERR " DDD   raw iPrice ===$iPrice===\n" if  (DEBUG_COLUMNS || (1 < $self->{_debug}));
   $iPrice =~ s!&pound;!GBP!;
+  $iPrice =~ s!\s*Trending.+!!;
+  $iPrice =~ s!\s*Was.+!!;
   # Convert nbsp to regular space:
   $iPrice =~ s!\240!\040!g;
   # I don't know why there are sometimes weird characters in there:
@@ -415,6 +446,7 @@ sub _parse_price
     {
     $hit->shipping('free');
     } # if
+  print STDERR " DDD   good final iPrice ===$iPrice===\n" if  (DEBUG_COLUMNS || (1 < $self->{_debug}));
   $hit->bid_amount($iPrice);
   return 1;
   } # _parse_price
@@ -427,6 +459,12 @@ sub _parse_bids
   my $iBids = 0;
   if (ref $oTDbids)
     {
+    if (my $oChild = $oTDbids->look_down(_tag => 'div',
+                                         class => 'bids'))
+      {
+      # As of 2013-03, we need to separate out the price and the bid:
+      $oTDbids = $oChild;
+      } # if
     my $s = $oTDbids->as_HTML;
     if (DEBUG_COLUMNS || (1 < $self->{_debug}))
       {
@@ -454,6 +492,12 @@ sub _parse_bids
       $iBids = 0;
       } # if
     } # if
+  if ($iBids =~ m/NO/i)
+    {
+    $iBids = 0;
+    } # if
+  $iBids ||= 0;
+  # print STDERR " DDD   setting bid_count to =$iBids=\n";
   $hit->bid_count($iBids);
   return 1;
   } # _parse_bids
@@ -470,12 +514,30 @@ sub _parse_shipping
     # into the list of international items.
     return 0;
     } # if
+  if (my $oChild = $oTD->look_down(_tag => 'span',
+                                   class => 'ship'))
+    {
+    # As of 2013-03, we need to separate out the price and the
+    # shipping for some flavors of eBay:
+    $oTD = $oChild;
+    } # if
   my $iPrice = $oTD->as_text;
   # I don't know why there are sometimes weird characters in there:
   $iPrice =~ s!&Acirc;!!g;
   $iPrice =~ s!!!g;
+  $iPrice =~ s!\+!!g;
+  $iPrice =~ s/SHIPPING//gi;
   print STDERR " DDD   raw shipping ===$iPrice===\n" if (DEBUG_COLUMNS || (1 < $self->{_debug}));
-  return 1 if ($iPrice !~ m/\d/);
+  if ($iPrice =~ m/UNKNOWN/i)
+    {
+    # No shipping info provided:
+    return 1;
+    } # if
+  if ($iPrice =~ m/FREE/i)
+    {
+    $iPrice = 0.00;
+    } # if
+  return 0 if ($iPrice !~ m/\d/);
   $iPrice =~ s!&pound;!GBP!;
   $hit->shipping($iPrice);
   return 1;
@@ -506,9 +568,45 @@ sub _parse_enddate
     $sDateTemp = $s = $oTDdate;
     }
   print STDERR " DDD   TDdate ===$s===\n" if (DEBUG_COLUMNS || (1 < $self->{_debug}));
+  # New version as of 2013-03:
+  if ($s =~ m/\bTIMEMS="(\d+)"/i)
+    {
+    $sDate = $1;
+    $sDate = $self->_format_date(ParseDate(q{epoch }. int($sDate/1000)));
+    print STDERR " DDD   sDate =$sDate=\n" if (DEBUG_COLUMNS || (1 < $self->{_debug}));
+    $hit->end_date($sDate);
+    # For backward-compatibility:
+    $hit->change_date($sDate);
+    return 1;
+    }
+  print STDERR " DDD   raw     sDateTemp ===$sDateTemp===\n" if (DEBUG_DATES || (1 < $self->{_debug}));
+  # I don't know why there are sometimes weird characters in there:
+  $sDateTemp =~ s!&Acirc;!!g;
+  $sDateTemp =~ s!!!g;
+  $sDateTemp =~ s!<!!;
+  $sDateTemp =~ tr!()!!d;
+  # Convert nbsp to regular space:
+  $sDateTemp =~ s!\240!\040!g;
+  $sDateTemp =~ s!Time\s+left:?\s*!!g;
+  $sDateTemp =~ s!\s+left!!g;
+  print STDERR " DDD   poached sDateTemp ===$sDateTemp===\n" if (DEBUG_DATES || (1 < $self->{_debug}));
+  my $date;
   if (ref($oTDdate))
     {
-    if ($oTDdate->attr('class') !~ m/\b(ebcTim|ti?me)\b/)
+    my $sClass = $oTDdate->attr('class') || q{};
+    if ($sClass =~ m/time-end/)
+      {
+      $date = Date_Parse($sDateTemp);
+      # Fall through and continue
+      }
+    elsif ($sClass =~ m/time(-(left|-soon))?/)
+      {
+      $sDateTemp = $self->_process_date_abbrevs($sDateTemp);
+      # print STDERR " DDD   official time =====$self->{_ebay_official_time}=====\n" if (DEBUG_DATES || (1 < $self->{_debug}));
+      $date = DateCalc($self->{_ebay_official_time}, " + $sDateTemp");
+      # Fall through and continue
+      }
+    elsif ($sClass !~ m/(col3|ebcTim|ti?me)/)
       {
       # If we see this, we probably were searching for Buy-It-Now items
       # but we ran off the bottom of the item list and ran into the list
@@ -516,8 +614,12 @@ sub _parse_enddate
       return 0;
       # There is a separate backend for searching Store items!
       } # if
+    else
+      {
+      warn " EEE did not match time/date class '$sClass'";
+      return 1;
+      }
     } # if
-  print STDERR " DDD   raw    sDateTemp ===$sDateTemp===\n" if (DEBUG_DATES || (1 < $self->{_debug}));
   if ($sDateTemp =~ m/---/)
     {
     # If we see this, we probably were searching for Buy-It-Now items
@@ -526,17 +628,7 @@ sub _parse_enddate
     return 0;
     # There is a separate backend for searching Store items!
     } # if
-  # I don't know why there are sometimes weird characters in there:
-  $sDateTemp =~ s!&Acirc;!!g;
-  $sDateTemp =~ s!!!g;
-  $sDateTemp =~ s!<!!;
-  # Convert nbsp to regular space:
-  $sDateTemp =~ s!\240!\040!g;
-  $sDateTemp =~ s!Time\s+left:!!;
-  $sDateTemp = $self->_process_date_abbrevs($sDateTemp);
-  print STDERR " DDD   cooked sDateTemp ===$sDateTemp===\n" if (DEBUG_DATES || (1 < $self->{_debug}));
-  print STDERR " DDD   official time =====$self->{_ebay_official_time}=====\n" if (DEBUG_DATES || (1 < $self->{_debug}));
-  my $date = DateCalc($self->{_ebay_official_time}, " + $sDateTemp");
+  print STDERR " DDD   cooked  sDateTemp ===$sDateTemp===\n" if (DEBUG_DATES || (1 < $self->{_debug}));
   print STDERR " DDD   date ===$date===\n" if (DEBUG_DATES || (1 < $self->{_debug}));
   $sDate = $self->_format_date($date);
   print STDERR " DDD   sDate ===$sDate===\n" if (DEBUG_DATES || (1 < $self->{_debug}));
@@ -565,7 +657,7 @@ sub result_as_HTML
   my $self = shift;
   my $oSR = shift or return '';
   my $sDateFormat = shift || q'%Y-%m-%d %H:%M:%S';
-  my $dateEnd = ParseDate($oSR->end_date);
+  my $dateEnd = ParseDate($oSR->end_date) || q{};
   my $iItemNum = $oSR->item_number;
   my $sSold = $oSR->sold
   ? $cgi->font({color=>'green'}, 'sold') .q{; }
@@ -577,7 +669,7 @@ sub result_as_HTML
   my $dateNow = ParseDate('now');
   print STDERR " DDD compare end_date ==$dateEnd==\n" if (DEBUG_DATES || (1 < $self->{_debug}));
   print STDERR " DDD compare date_now ==$dateNow==\n" if (DEBUG_DATES || (1 < $self->{_debug}));
-  if (Date_Cmp($dateEnd, $dateNow) < 0)
+  if ((Date_Cmp($dateEnd, $dateNow) || 0) < 0)
     {
     $sEndedColor = 'red';
     $sEndedWord = 'ended';
@@ -624,26 +716,44 @@ sub _get_result_count_elements
   {
   my $self = shift;
   my $tree = shift;
-  my @ao = $tree->look_down(
-                            '_tag' => 'div',
-                            class => 'fpcc'
-                           );
+  my @ao;
+  push @ao, $tree->look_down( # as of 2015-06
+                             '_tag' => 'span',
+                             class => 'listingscnt'
+                            );
+  push @ao, $tree->look_down(
+                             '_tag' => 'div',
+                             class => 'fpcc'
+                            );
   push @ao, $tree->look_down(
                              '_tag' => 'div',
                              class => 'fpc'
                             );
+  push @ao, $tree->look_down(
+                             # For basic search, as of 2013-03:
+                             '_tag' => 'div',
+                             class => 'clt'
+                            );
   push @ao, $tree->look_down(
                             '_tag' => 'div',
                             class => 'count'
                            );
   push @ao, $tree->look_down(
-                            '_tag' => 'div',
-                            class => 'pageCaptionDiv'
-                           );
+                             '_tag' => 'div',
+                             class => 'pageCaptionDiv'
+                            );
   push @ao, $tree->look_down( # for BySellerID as of 2010-07
-                            '_tag' => 'div',
-                            id => 'rsc'
-                           );
+                             '_tag' => 'div',
+                             id => 'rsc'
+                            );
+  push @ao, $tree->look_down( # for Category, as of 2018-02:
+                             _tag => 'h2',
+                             class => 'srp-controls__count-heading'
+                            );
+  push @ao, $tree->look_down( # new as of 2018-02:
+                             _tag => 'h1',
+                             class => 'srp-controls__count-heading'
+                            );
   return @ao;
   } # _get_result_count_elements
 
@@ -674,6 +784,21 @@ sub _get_itemtitle_tds
   push @ao, $tree->look_down(_tag => 'td',
                              class => 'details ttl',
                             );
+  my $oDiv = $tree->look_down(_tag => 'div',
+                              id => 'ResultSetItems',
+                             );
+  if (ref $oDiv)
+    {
+    push @ao, $oDiv->look_down(_tag => 'td',
+                               class => 'dtl dtlsp',
+                              );
+    push @ao, $oDiv->look_down(_tag => 'h3',
+                               class => 'lvtitle',
+                              );
+    } # if
+  push @ao, $tree->look_down(_tag => 'div',
+                             class => 's-item__info clearfix',
+                            );
   return @ao;
   } # _get_itemtitle_tds
 
@@ -704,7 +829,7 @@ sub _parse_tree
     $self->approximate_result_count(1);
     return 1;
     } # if
-  my $iHits = 0;
+
   # First, see if: there were zero results and eBay automatically did
   # a spell-check and searched for other words (or searched for a
   # subset of query terms):
@@ -729,6 +854,7 @@ sub _parse_tree
       return 0;
       } # if
     } # if
+
   # See if our query was completely replaced by a similar-spelling query:
   my $oLI = $tree->look_down(_tag => 'li',
                              class => 'ebInf',
@@ -741,24 +867,49 @@ sub _parse_tree
       return 0;
       } # if
     } # if
-  # The hit count is in a FONT tag:
-  my @aoFONT = $self->_get_result_count_elements($tree);
- FONT:
-  foreach my $oFONT (@aoFONT)
+
+  # See if our category-only query was replaced by a global query:
+  my $oP = $tree->look_down(_tag => 'p',
+                             class => 'sm-md',
+                            );
+  if (ref $oP)
     {
+    my $s = $oP->as_text;
+    if (($s =~ m/0 results found in the/) && ($s =~ m/so we searched in all categories/))
+      {
+      return 0;
+      } # if
+    } # if
+
+  my $iHits = 0;
+
+  # The hit count is in one of these tags:
+  my @aoResultCountTagset = $self->_get_result_count_elements($tree);
+  if (scalar(@aoResultCountTagset) < 1)
+    {
+    warn " EEE no result_count_elements matched the HTML\n";
+    } # if
+ FONT:
+  foreach my $oFONT (@aoResultCountTagset)
+    {
+    my $s = $oFONT->as_text;
     my $qr = $self->_result_count_pattern;
-    print STDERR (" DDD   result_count try ==",
-                  $oFONT->as_text, "== against qr=$qr=\n") if (1 < $self->{_debug});
-    if ($oFONT->as_text =~ m!$qr!)
+    print STDERR (" DDD   result_count try qr=$qr= against ==$s==\n") if (1 < $self->{_debug});
+    if ($s =~ m/$qr/)
       {
       my $sCount = $1;
       print STDERR " DDD     matched ($sCount)\n" if (1 < $self->{_debug});
       # Make sure it's an integer:
       $sCount =~ s!,!!g;
-      $self->approximate_result_count($sCount);
+      $self->approximate_result_count(0 + $sCount);
       last FONT;
       } # if
-    } # foreach
+    } # foreach FONT
+
+  if ($self->approximate_result_count() < 1)
+    {
+    return $iHits;
+    } # if
 
   # Recursively parse the stats telling how many items were found in
   # each category:
@@ -777,32 +928,6 @@ sub _parse_tree
     $oDiv->detach;
     $oDiv->delete;
     } # if
-  # By default, use the hard-coded order of columns:
-  my @asColumns = $self->_columns;
-  if (ref($self) !~ m!::Completed!)
-    {
-    # See if we can glean the actual order of columns from the page itself:
-    my @aoCOL = $tree->look_down(_tag => 'col');
-    my @asId;
-    foreach my $oCOL (@aoCOL)
-      {
-      # Sanity check:
-      next unless ref($oCOL);
-      my $sId = $oCOL->attr('id') || '';
-      # Sanity check:
-      next unless ($sId ne '');
-      $sId =~ s!\Aebc!!;
-      # Try not to go past the first table:
-      last if ($sId eq 'bdrRt');
-      push @asId, $sId;
-      } # foreach
-    print STDERR " DDD raw    asId is (@asId)\n" if (1 < $self->{_debug});
-    1 while (@asId && (shift(@asId) ne 'title'));
-    local $" = ',';
-    print STDERR " DDD cooked asId is (@asId)\n" if (1 < $self->{_debug});
-    @asColumns = @asId if @asId;
-    } # if
-  print STDERR " DDD   asColumns is (@asColumns)\n" if (1 < $self->{_debug});
   # The list of matching items is in a table.  The first column of the
   # table is nothing but icons; the second column is the good stuff.
   my @aoTD = $self->_get_itemtitle_tds($tree);
@@ -829,11 +954,11 @@ sub _parse_tree
     print STDERR " DDD   sTitle ===$sTitle===\n" if (1 < $self->{_debug});
     my $oURI = URI->new($oA->attr('href'));
     # next TD unless ($oURI =~ m!ViewItem!);
-    next TD unless ($oURI =~ m!$qrItemNum!);
+    next TD if ($oURI !~ m!$qrItemNum!);
     my $iItemNum = $1;
+    print STDERR " DDD   iItemNum ===$iItemNum===\n" if (1 < $self->{_debug});
     my $iCategory = -1;
     $iCategory = $1 if ($oURI =~ m!QQcategoryZ(\d+)QQ!);
-    print STDERR " DDD   iItemNum ===$iItemNum===\n" if (1 < $self->{_debug});
     if ($oURI->as_string =~ m!QQitemZ(\d+)QQ!)
       {
       # Convert new eBay links to old reliable ones:
@@ -845,86 +970,67 @@ sub _parse_tree
     my $hit = new WWW::Search::Result;
     $hit->add_url($self->_cleanup_url($sURL));
     $hit->title($sTitle);
-    $hit->category($iCategory);
     $hit->item_number($iItemNum);
-    # The rest of the info about this item is in sister TD elements to
-    # the right:
-    my @aoSibs = $oTDtitle->right;
-    # But in the Completed auctions list, the rest of the info is in
-    # the next row of the table:
-    if (0 && ref($self) =~ m!::Completed!)
-      {
-      @aoSibs = ();
-      my $oTRparent = $oTDtitle->look_up(_tag => 'tr');
-      if (ref $oTRparent)
-        {
-        my $sTRparent = $oTRparent->as_HTML;
-        # print STDERR " DDD oTRparent ==$sTRparent==\n";
-        my $oTRaunt = $oTRparent->right;
-        if (ref $oTRaunt)
-          {
-          my $sTRaunt = $oTRaunt->as_HTML;
-          # print STDERR " DDD oTRaunt ==$sTRaunt==\n";
-          @aoSibs = $oTRaunt->look_down(_tag => 'td');
-          # Throw out one empty cell:
-          shift @aoSibs;
-          } # if
-        } # if
-      } # if
-    my $iCol = 0;
+    # This is just to prevent undef warnings later on:
+    $hit->bid_count(0);
+    # The rest of the info about this item is in sister <LI> elements
+    # to the right:
+    my @aoSibs = $oTDtitle->look_down(_tag => q/span/);
+    push @aoSibs, $oTDtitle->parent->look_down(_tag => q/li/);
+    # The parent itself is an <LI> tag:
+    shift @aoSibs;
+    warn " DDD before loop, there are ", scalar(@aoSibs), " sibling TDs\n" if (1 < $self->{_debug});
  SIBLING_TD:
-    while ((my $oTDsib = shift(@aoSibs))
-           &&
-           (my $sColumn = $asColumns[$iCol++])
-          )
+    while (my $oTDsib = shift @aoSibs)
       {
       next unless ref($oTDsib);
+      my $sColumn = $oTDsib->attr('class') || q{};
       my $s = $oTDsib->as_HTML;
-      print STDERR " DDD   try TD'$sColumn' ===$s===\n" if (DEBUG_COLUMNS || (1 < $self->{_debug}));
-      if ($sColumn eq 'price')
+      if ($sColumn eq q{})
+        {
+        warn " WWW auction info sibling has no class ==$s==" if (DEBUG_COLUMNS || (1 < $self->{_debug}));
+        } # if
+      print STDERR " DDD   looking at sibling TD whose class is '$sColumn' ===$s===\n" if (DEBUG_COLUMNS || (1 < $self->{_debug}));
+      if ($sColumn =~ m'price')
         {
         next TD unless $self->_parse_price($oTDsib, $hit);
-        }
-      elsif ($sColumn eq 'bids')
+        } # if
+      if (($sColumn =~ m'bids') || ($sColumn =~ m'format'))
         {
-        next TD unless $self->_parse_bids($oTDsib, $hit);
+        # It is not a fatal error if there are no bids (i.e. buy-it-now)
+        $self->_parse_bids($oTDsib, $hit);
         }
-      elsif ($sColumn eq 'shipping')
+      if ($sColumn =~ m'shipping')
         {
-        next TD unless $self->_parse_shipping($oTDsib, $hit);
+        next TD if ! $self->_parse_shipping($oTDsib, $hit);
         }
-      elsif ($sColumn eq 'enddate')
+      if ($sColumn =~ m'end')
         {
-        next TD unless $self->_parse_enddate($oTDsib, $hit);
+        next TD if ! $self->_parse_enddate($oTDsib, $hit);
         }
-      elsif ($sColumn eq 'time')
+      if ($sColumn =~ m'time')
         {
-        next TD unless $self->_parse_enddate($oTDsib, $hit);
+        next TD if ! $self->_parse_enddate($oTDsib, $hit);
         }
-      elsif ($sColumn eq 'country')
+      if ($sColumn =~ m'country')
         {
         # This listing is from a country other than the base site
         # we're searching against.  Throw it out:
         next TD;
         }
-      elsif ($sColumn eq 'paypal')
+      if ($sColumn =~ m'extras')
         {
-        # We always ignore the Paypal logo.
-        next SIBLING_TD;
-        }
-      elsif ($sColumn eq 'buyitnowlogo')
-        {
-        # We always ignore the Buy-It-Now logo.
-        next SIBLING_TD;
-        }
-      else
-        {
-        print STDERR " DDD     do not know how to handle column named $sColumn\n" if (1 < $self->{_debug});
-        next SIBLING_TD;
-        }
+        if ($iCategory < 0)
+          {
+          # We haven't found this item's category.  Look for it here:
+          $iCategory = $self->_parse_category($oTDsib);
+          } # if
+        } # if 'extras'
+      # Any other class="" value will cause the <LI> to be ignored.
       } # while
     my $sDesc = $self->_create_description($hit);
     $hit->description($sDesc);
+    $hit->category($iCategory);
     # Clean up / sanity check hit info:
     my ($enddate, $iBids);
     if (
@@ -934,15 +1040,25 @@ sub _parse_tree
         &&
         (0 < $iBids) # Item got any bids
         &&
-        (Date_Cmp($enddate, 'now') < 0) # Item is ended
+        ((Date_Cmp($enddate, 'now') || 0) < 0) # Item is ended
        )
       {
       # Item must have been sold!?!
       $hit->sold(1);
       } # if
+    print STDERR " DDD   add hit to cache\n" if (1 < $self->{_debug});
     push(@{$self->{cache}}, $hit);
     $self->{'_num_hits'}++;
     $iHits++;
+
+    if ($self->approximate_result_count() <= $iHits)
+      {
+      # We've already scraped all the hits that eBay said there were.
+      # Stop scraping, so that we don't return hits that are not an
+      # exact match to the query but are just "related":
+      last TD;
+      } # if
+
     # Delete this HTML element so that future searches go faster?
     $oTDtitle->detach;
     $oTDtitle->delete;
@@ -954,7 +1070,7 @@ sub _parse_tree
     # AS OF 2008-11 THE NEXT LINK CAN NOT BE FOLLOWED FROM PERL CODE
 
     # Look for a NEXT link:
-    my @aoA = $tree->look_down('_tag', 'a');
+    my @aoA = $tree->look_down('_tag' => 'a');
  TRY_NEXT:
     foreach my $oA (0, reverse @aoA)
       {
@@ -1124,23 +1240,10 @@ Include parentheses so that $1 becomes the number (with commas is OK).
 
 sub _result_count_pattern
   {
-  return qr'([0-9,]+) (item|match|result)e?s? found\b';
+  return qr'([0-9,]+)\s+(active\s+)?(listing|item|matche?|result)s?(\s+found)?'i;
   } # _result_count_pattern
 
 
-=item _columns
-
-Specify the order in which data columns appear in the search results table.
-
-=cut
-
-sub _columns
-  {
-  my $self = shift;
-  # This is for basic USA eBay:
-  return qw( paypal bids price enddate );
-  } # _columns
-
 1;
 
 __END__
